// ===== Modellierung ========== Wolfgang Erben

package studienarbeit;

/**
 * Die Schnittstelle beschreibt KurvenBerechnungen.
 * Es muss gewaehrleistet sein, dass die entsprechenden Toleranzen
 * fuer Kurven mit bis zu 10m Gesamtlaenge eingehalten werden.
 * 
 * @author Christian Siegle
 *
 */
public interface IKurvenBerechnung {
    
     /**
     * Laengen-Toleranz in mm. 
     * Diese gibt an, ab welcher Differenz zwei Laengen als gleich zu betrachten
     * sind. 
     */
    double LAENGEN_TOLERANZ = 0.02;
    
    /**
     * Flaechen-Toleranz in mm^2. 
     * Diese gibt an, ab welcher Differenz zwei Flaechen als gleich
     * zu betrachten sind. 
     */
    double FLAECHEN_TOLERANZ = 0.2;
    
    /**
     * Punkt-Toleranz in mm. 
     * Diese gibt an, ab welchem Abstand zwei Punkte als gleich zu betrachten
     * sind. 
     */
    double PUNKT_TOLERANZ = 0.02;
    
    
    /**
     * initialisiert die KurvenBerechnung mit der angegebenen Kurve.
     */
    void initialisierenFuer(IKurve kurve);
    
    /**
     * liefert die Laenge der angegebenen Kurve.
     */
    double laenge();
    
    /**
     * liefert den Flaecheninhalt der angegebenen Kurve.
     */
    double flaecheninhalt();
    
    /**
     * liefert den Schwerpunkt der angegebenen Kurve.
     */
    IPunkt schwerpunkt();
    
}