// ===== Modellierung ========== Wolfgang Erben

package pruefung2;

import java.util.ArrayList;
import java.util.List;

import pflichtuebung1.korrekt.Intervall;
import studienarbeit.IKurve;
import studienarbeit.IPunkt;

/**
 * beschreibt ein ebenes Polygon.
 * 
 * @author Wolfgang Erben
 * @version 1.0 20.05.2010
 */
public class Polygon implements IKurve {
// TODO Die Klasse Polygon muss noch vervollstaendigt werden 
    private final List<IPunkt> mPunkte = new ArrayList<IPunkt>();

    /**
     * erzeugt ein Polygon aus den angegebenen Punkten.
     */
    public Polygon(IPunkt... punkte) {
        for (IPunkt p : punkte) mPunkte.add(p);
    }
    
    /**
     * erzeugt ein Polygon aus der angegebenen Kurve durch
     * aequidistante Aufteilung des Parameterintervalls in die
     * angegebene Anzahl von Teilen. 
     */
    public Polygon(IKurve kurve, int anzahl) {
        final double dt = 1. / anzahl;
        for (int i = 0; i < anzahl; i++) {
            double t = i * dt;
            IPunkt p = Kurven.kurvenPunkt(kurve, t);
            mPunkte.add(p);
        }
        IPunkt p = Kurven.kurvenPunkt(kurve, 1.);
        mPunkte.add(p);
    }
    /**
     * liefert die Anzahl der Punkte des Polygons.
     */
    public int anzahlPunkte() {
        return mPunkte.size();
    }
    /**
     * liefert einen einzelnen Punkt des Polygons.
     * Der erste Punkt hat die Nummer 0.
     */
    public IPunkt punktNr(int nr) {
        return mPunkte.get(nr);
    }
    /**
     * liefert eine einzelne Strecke des Polygons. 
     * Die erste Strecke hat die Nummer 1. 
     */
    private Strecke streckeNr(int nr) {
        IPunkt a = mPunkte.get(nr - 1);
        IPunkt b = mPunkte.get(nr);
        return new Strecke(a, b);
    }
    /**
     * liefert die zum angegebenen Parameterwert
     * gehoerende Strecke des Polygons. 
     */
    private Strecke streckeZu(double t) {
        int nr = streckenNummerZu(t);
        return streckeNr(nr);
    }
    /**
     * liefert die Nummer der Strecke zum angegebenen 
     * Parameterwert. Die erste Strecke hat die Nummer 1.
     */
    private int streckenNummerZu(double t) {
        int teile = mPunkte.size() - 1;
        if (t == 1.) return teile;
        double tGesamt = t * teile;
        int nr = (int) tGesamt;
        return nr + 1;
    }
    /**
     * liefert den lokalen Parameterwert, also bezogen auf die
     * zugehoerige Strecke, zum angegebenen globalen Parameterwert. 
     */
    private double lokalerParameterZu(double t) {
        if (t == 1.) return 1.;
        int teile = mPunkte.size() - 1;
        double tGesamt = t * teile;
        int nr = (int) tGesamt;
        return tGesamt - nr;
    }
    /**
     * liefert die x-Koordinate des Punktes zum angegebenen Parameter-Wert.
     */
    @Override
    public double x(double t) {
        Strecke s = streckeZu(t);
        double tLokal = lokalerParameterZu(t);
        return s.x(tLokal);
    }
    /**
     * liefert das Bild des angegebenen Intervalls durch die Funktion x(t),
     * also alle moeglichen x-Werte, wenn t das angegebene Intervall 
     * durchlaeuft. 
     */
    @Override
    public Intervall x(Intervall urbild) {
// TODO Funktion x(Intervall) noch etwas konkretisieren
        throw new UnsupportedOperationException();
    }
    /**
     * liefert die y-Koordinate des Punktes zum angegebenen Parameter-Wert.
     */
    @Override
    public double y(double t) {
        Strecke s = streckeZu(t);
        double tLokal = lokalerParameterZu(t);
        return s.y(tLokal);
    }
    /**
     * liefert das Bild des angegebenen Intervalls durch die Funktion y(t),
     * also alle moeglichen y-Werte, wenn t das angegebene Intervall 
     * durchlaeuft.
     */
    @Override
    public Intervall y(Intervall urbild) {
// TODO Funktion y(Intervall) noch etwas konkretisieren
        throw new UnsupportedOperationException();
    }
    /**
     * liefert eine textuelle Darstellung des Polygons.
     */
    @Override
    public String toString() {
        return "Polygon" + mPunkte;
    }
    /**
     * liefert die Laenge des Polygons.
     */
    public double laenge() {
        double laenge = 0.;
        for (int i = 1; i < mPunkte.size(); i++) {
            Strecke s = streckeNr(i);
            laenge += s.laenge();
        }
        return laenge;
    }
}
