// ===== Modellierung ========== Wolfgang Erben

package pruefung2;

import pflichtuebung1.korrekt.Intervall;
import studienarbeit.*;

/**
 * beschreibt eine Strecke im R2.
 * 
 * @author Wolfgang Erben
 * @version 1.0 20.05.2010
 */
public class Strecke implements IKurve {
    private final IPunkt mVon, mBis;

    /**
     * erzeugt eine Strecke vom ersten zum zweiten
     * angegebenen Punkt. 
     */
    public Strecke(IPunkt von, IPunkt bis) {
        mVon = von;
        mBis = bis;
    }
    /**
     * liefert die x-Koordinate des Punktes zum angegebenen Parameter-Wert.
     */
    @Override
    public double x(double t) {
        return mVon.x() * (1. - t) + mBis.x() * t;
    }
    /**
     * liefert das Bild des angegebenen Intervalls durch die Funktion x(t),
     * also alle moeglichen x-Werte, wenn t das angegebene Intervall 
     * durchlaeuft. 
     */
    @Override
    public Intervall x(Intervall urbild) {
        double x1 = x(urbild.untereGrenze());
        double x2 = x(urbild.obereGrenze());
        double xu = Math.min(x1, x2);
        double xo = Math.max(x1, x2);
        return new Intervall(xu, xo);
    }
    /**
     * liefert die y-Koordinate des Punktes zum angegebenen Parameter-Wert.
     */
    @Override
    public double y(double t) {
        return mVon.y() * (1. - t) + mBis.y() * t;
    }
    /**
     * liefert das Bild des angegebenen Intervalls durch die Funktion y(t),
     * also alle moeglichen y-Werte, wenn t das angegebene Intervall 
     * durchlaeuft.
     */
    @Override
    public Intervall y(Intervall urbild) {
        double y1 = y(urbild.untereGrenze());
        double y2 = y(urbild.obereGrenze());
        double yu = Math.min(y1, y2);
        double yo = Math.max(y1, y2);
        return new Intervall(yu, yo);
    }
    /**
     * liefert eine textuelle Darstellung der Strecke.
     */
    @Override
    public String toString() {
        return "Strecke[" + mVon + "->" + mBis + "]";
    }
    /**
     * liefert die Laenge der Strecke.
     */
    public double laenge() {
        return mVon.abstandZu(mBis);
    }
    /**
     * liefert den Mittelpunkt der Strecke.
     */
    public IPunkt mitte() {
        double x = mVon.x() / 2. + mBis.x() / 2.;
        double y = mVon.y() / 2. + mBis.y() / 2.;
        return new Punkt(x, y);
    }

}
