// ===== Modellierung ========== Wolfgang Erben

package pruefung2.hilfsmittel;

import static org.junit.Assert.*;

import org.junit.Test;

import pflichtuebung1.korrekt.Intervall;
import pruefung2.*;
import studienarbeit.*;
import static pruefung2.Kurven.*;
import static studienarbeit.IKurve.INTERVALL;

/**
 * testet die Klasse Polygon.
 * 
 * @author Wolfgang Erben
 * @version 1.0 03.06.2010
 */
public class TestPolygon {
    private static final double PEANUTS = 1.e-5;
    private static final int INTEGRATIONEN = 1000;
    private static final double VIERTELE = 0.25;
    /**
     * testet achsensymmetrische Rechtecke.
     */
    @Test
    public void testeSymmetrischeRechtecke() {
        for (int i = 0; i < INTEGRATIONEN; i++)
            testeSymmetrischeRechteckeEinmal();
    }
    private void testeSymmetrischeRechteckeEinmal() {
        double x = Kurven.zufaelligePositiveKoordinate();
        double y = Kurven.zufaelligePositiveKoordinate();
        Punkt p1 = new Punkt(x, y);
        Punkt p2 = new Punkt(-x, y);
        Punkt p3 = new Punkt(-x, -y);
        Punkt p4 = new Punkt(x, -y);
        Punkt[] punkte = {p1, p2, p3, p4, p1};
        Polygon p = new Polygon(punkte);
        assertEquals(punkte.length, p.anzahlPunkte());
        for (int i = 0; i < punkte.length; i++)
            assertEquals(punkte[i], p.punktNr(i));
        Polygon q = new Polygon(punkte);
        assertEquals(p.toString(), q.toString());
        assertFastGleich(p.laenge(), q.laenge());
        assertEquals(p1, kurvenPunkt(p, 0.));
        assertFastGleich(p2, kurvenPunkt(p, VIERTELE));
        assertFastGleich(p3, kurvenPunkt(p, 0.5));
        assertFastGleich(p4, kurvenPunkt(p, 0.75));
        assertEquals(p1, kurvenPunkt(p, 1.));
        assertFastGleich(p1.mitteZu(p2), kurvenPunkt(p, 0.125));
        assertFastGleich(p2.mitteZu(p3), kurvenPunkt(p, 0.375));
        assertFastGleich(p3.mitteZu(p4), kurvenPunkt(p, 0.625));
        assertFastGleich(p4.mitteZu(p1), kurvenPunkt(p, 0.875));
        assertFastGleich(4. * (x + y), p.laenge());
    }
    private static void assertFastGleich(double d1, double d2) {
        assertEquals(d1, d2, PEANUTS);
    }
    private static void assertFastGleich(IPunkt p1, IPunkt p2) {
        if (p1 == p2) return;
        double d = p1.abstandZu(p2);
        assertTrue(d < PEANUTS);
    }
    /** 
     * testet die Erzeugung eine Polygons aus Kurven.
     */
    @Test
    public void testeErzeugungAusKurven() {
        for (int i = 0; i < INTEGRATIONEN; i++)
            testeErzeugungAusKurvenEinmal();
    }
    private void testeErzeugungAusKurvenEinmal() {
        Punkt[] punkte = {zufaelligerPunkt(), zufaelligerPunkt(),
                            zufaelligerPunkt()};
        Polygon p = new Polygon(punkte);
        Polygon q1 = new Polygon(p, punkte.length - 1);
        assertEquals(punkte.length, q1.anzahlPunkte());
        assertEquals(kurvenPunkt(p, 0.), kurvenPunkt(q1, 0.));
        assertEquals(kurvenPunkt(p, 1.), kurvenPunkt(q1, 1.));
        double t = Math.random();
        assertFastGleich(kurvenPunkt(p, t), kurvenPunkt(q1, t));
    }
    /**
     * testet die Methoden zum Abbilden von Intervallen.
     */
    @Test
    public void testeAbbildungVonIntervallen() {
        Punkt p1 = new Punkt(-100., 0.);
        Punkt p2 = new Punkt(0., 100.);
        Punkt p3 = new Punkt(100., 0.);
        Strecke s1 = new Strecke(p1, p2);
        Strecke s2 = new Strecke(p2, p3); 
        Polygon k = new Polygon(p1, p2, p3);
        assertEquals(kurvenPunkt(s1, 0.), kurvenPunkt(k, 0.));
        assertEquals(kurvenPunkt(s1, 1.), kurvenPunkt(k, 0.5));
        assertEquals(kurvenPunkt(s2, 0.), kurvenPunkt(k, 0.5));
        assertEquals(kurvenPunkt(s2, 1.), kurvenPunkt(k, 1.));
        assertEquals(kurvenPunkt(s1, 0.5), kurvenPunkt(k, 0.25));
        assertEquals(kurvenPunkt(s2, 0.5), kurvenPunkt(k, 0.75));
        assertEquals(s1.x(0.1), k.x(0.05), PEANUTS);
        assertEquals(s1.y(0.1), k.y(0.05), PEANUTS);
        assertEquals(s2.x(0.1), k.x(0.55), PEANUTS);
        assertEquals(s2.y(0.1), k.y(0.55), PEANUTS);
        assertEquals(s1.y(INTERVALL), k.y(INTERVALL));
        assertEquals(s2.y(INTERVALL), k.y(INTERVALL));
        Intervall u = INTERVALL.untereHaelfte();
        Intervall o = INTERVALL.obereHaelfte();
        assertEquals(s1.x(INTERVALL), k.x(u));
        assertEquals(s2.x(INTERVALL), k.x(o));
        assertEquals(s1.x(u), k.x(u.untereHaelfte()));
        assertEquals(s1.y(u), k.y(u.untereHaelfte()));
        assertEquals(s1.x(o), k.x(u.obereHaelfte()));
        assertEquals(s1.y(o), k.y(u.obereHaelfte()));
        assertEquals(s2.x(u), k.x(o.untereHaelfte()));
        assertEquals(s2.y(u), k.y(o.untereHaelfte()));
        assertEquals(s2.x(o), k.x(o.obereHaelfte()));
        assertEquals(s2.y(o), k.y(o.obereHaelfte()));
        
    }

}
