// ===== Modellierung ========== Wolfgang Erben

package pruefung2;

import pflichtuebung1.korrekt.Intervall;
import studienarbeit.IKurve;

/**
 * beschreibt eine Kurve, welche durch Verkettung zweier
 * anderer Kurven entsteht. 
 * 
 * @author Wolfgang Erben
 * @version 1.0 15.06.2010
 */
public class Verkettung implements IKurve {
    private final IKurve mVorne, mHinten;
    private static final double MITTE = 1. / 2.; 
    /**
     * erzeugt eine Kurve als Verkettung der angegebenen Kurven
     * in der angegebenen Reihenfolge. 
     * Die erste Kurve hat in der globalen Parametrisierung 
     * das Parameter-Intervall [0,1/2], die zweite das Intervall [1/2,1]. 
     */
    public Verkettung(IKurve vorne, IKurve hinten) {
       mVorne = vorne;
       mHinten = hinten;
    }
    /**
     * liefert die x-Koordinate des Punktes zum angegebenen Parameter-Wert.
     */
    @Override
    public double x(double t) {
        IKurve k = kurveZu(t);
        double tl = lokalerParameterZu(t);
        return k.x(tl);
    }
    /**
     * liefert diejenige der beiden Kurven, die zum angegebenen Parameterwert
     * gehoert.  
     */
    private IKurve kurveZu(double t) {
        if (t <= MITTE) return mVorne;
        return mHinten;
    }
    /**
     * liefert den lokalen Parameterwert, also bezogen auf die
     * zugehoerige Kurve, zum angegebenen globalen Parameterwert. 
     */
    private double lokalerParameterZu(double t) {
        if (t <= MITTE) return 2. * t;
        return 2. * t - 1.;
    }
    /**
     * liefert das Bild des angegebenen Intervalls durch die Funktion x(t),
     * also alle moeglichen x-Werte, wenn t das angegebene Intervall 
     * durchlaeuft. 
     * Es kann auch eine Obermenge hiervon zurueck gegeben werden, falls die
     * Berechnung des genauen Bildes zu schwierig ist. Es muss aber garantiert
     * sein, dass die Laenge des Intervalls gegen 0 geht, wenn dies die
     * Laenge des Urbildes tut. 
     */
    @Override
    public Intervall x(Intervall urbild) {
        Intervall lokal1 = lokalesIntervall1Zu(urbild);
        if (lokal1.istTeilmengeVon(INTERVALL)) return mVorne.x(lokal1);
        Intervall lokal2 = lokalesIntervall2Zu(urbild);
        if (lokal2.istTeilmengeVon(INTERVALL)) return mHinten.x(lokal2);
        lokal1 = lokal1.schnittMit(INTERVALL);
        lokal2 = lokal2.schnittMit(INTERVALL);
        Intervall erg1 = mVorne.x(lokal1);
        Intervall erg2 = mHinten.x(lokal2);
        return Intervall.konvexeHuelleVon(erg1, erg2);
    }
    private Intervall lokalesIntervall1Zu(Intervall global) {
        double a = 2. * global.untereGrenze();
        double b = 2. * global.obereGrenze();
        return new Intervall(a, b);
    }
    private Intervall lokalesIntervall2Zu(Intervall global) {
        double a = 2. * global.untereGrenze() - 1.;
        double b = 2. * global.obereGrenze() - 1.;
        return new Intervall(a, b);
    }
    /**
     * liefert die y-Koordinate des Punktes zum angegebenen Parameter-Wert.
     */
    @Override
    public double y(double t) {
        IKurve k = kurveZu(t);
        double tl = lokalerParameterZu(t);
        return k.y(tl);
    }
    /**
     * liefert das Bild des angegebenen Intervalls durch die Funktion y(t),
     * also alle moeglichen y-Werte, wenn t das angegebene Intervall 
     * durchlaeuft.
     * Es kann auch eine Obermenge hiervon zurueck gegeben werden, falls die
     * Berechnung des genauen Bildes zu schwierig ist. Es muss aber garantiert
     * sein, dass die Laenge des Intervalls gegen 0 geht, wenn dies die
     * Laenge des Urbildes tut. 
     */
    @Override
    public Intervall y(Intervall urbild) {
        Intervall lokal1 = lokalesIntervall1Zu(urbild);
        if (lokal1.istTeilmengeVon(INTERVALL)) return mVorne.y(lokal1);
        Intervall lokal2 = lokalesIntervall2Zu(urbild);
        if (lokal2.istTeilmengeVon(INTERVALL)) return mHinten.y(lokal2);
        lokal1 = lokal1.schnittMit(INTERVALL);
        lokal2 = lokal2.schnittMit(INTERVALL);
        Intervall erg1 = mVorne.y(lokal1);
        Intervall erg2 = mHinten.y(lokal2);
        return Intervall.konvexeHuelleVon(erg1, erg2);
    }

}
